@@ -166,6 +166,18 @@ if_setflag(struct interface *ifp, short flag)
 	return r;
 }
 
+static int
+if_hasconf(struct dhcpcd_ctx *ctx, const char *ifname)
+{
+	int i;
+
+	for (i = 0; i < ctx->ifcc; i++) {
+		if (strcmp(ctx->ifcv[i], ifname) == 0)
+			return 1;
+	}
+	return 0;
+}
+
 struct if_head *
 if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 {
@@ -283,6 +295,13 @@ if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 		if (!dev_initialized(ctx, p))
 			continue;
 
+		/* Don't allow loopback or pointopoint unless explicit */
+		if (ifa->ifa_flags & (IFF_LOOPBACK | IFF_POINTOPOINT)) {
+			if ((argc == 0 || argc == -1) &&
+			    ctx->ifac == 0 && !if_hasconf(ctx, p))
+				continue;
+		}
+
 		if (if_vimaster(p) == 1) {
 			syslog(argc ? LOG_ERR : LOG_DEBUG,
 			    "%s: is a Virtual Interface Master, skipping", p);
@@ -300,13 +319,7 @@ if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 		ifp->carrier = if_carrier(ifp);
 
 		sdl_type = 0;
-		/* Don't allow loopback unless explicit */
-		if (ifp->flags & IFF_LOOPBACK) {
-			if ((argc == 0 || argc == -1) && ctx->ifac == 0) {
-				if_free(ifp);
-				continue;
-			}
-		} else if (ifa->ifa_addr != NULL) {
+		if (ifa->ifa_addr != NULL) {
 #ifdef AF_LINK
 			sdl = (const struct sockaddr_dl *)(void *)ifa->ifa_addr;
 
@@ -335,8 +348,9 @@ if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 #ifdef IFT_BRIDGE
 			case IFT_BRIDGE:
 				/* Don't allow bridge unless explicit */
-				if ((argc == 0 || argc == -1)
-				    && ctx->ifac == 0)
+				if ((argc == 0 || argc == -1) &&
+				    ctx->ifac == 0 &&
+				    !if_hasconf(ctx, ifp->name))
 				{
 					if_free(ifp);
 					continue;
@@ -384,24 +398,29 @@ if_discover(struct dhcpcd_ctx *ctx, int argc, char * const *argv)
 #endif
 
 		/* We only work on ethernet by default */
-		if (!(ifp->flags & IFF_POINTOPOINT) &&
-		    ifp->family != ARPHRD_ETHER)
-		{
-			if ((argc == 0 || argc == -1) && ctx->ifac == 0) {
+		if (ifp->family != ARPHRD_ETHER) {
+			if ((argc == 0 || argc == -1) &&
+			    ctx->ifac == 0 && !if_hasconf(ctx, ifp->name))
+			{
 				if_free(ifp);
 				continue;
 			}
 			switch (ifp->family) {
 			case ARPHRD_IEEE1394: /* FALLTHROUGH */
 			case ARPHRD_INFINIBAND:
+#ifdef ARPHRD_LOOPBACK
+			case ARPHRD_LOOPBACK:
+#endif
+#ifdef ARPHRD_PPP
+			case ARPHRD_PPP:
+#endif
 				/* We don't warn for supported families */
 				break;
 			default:
 				syslog(LOG_WARNING,
-				    "%s: unsupported interface type %.2x"
-				    ", falling back to ethernet",
-				    ifp->name, sdl_type);
-				ifp->family = ARPHRD_ETHER;
+				    "%s: unsupported interface type %.2x, "
+				    "family %.2x",
+				    ifp->name, sdl_type, ifp->family);
 				break;
 			}
 		}
